@@ -2,6 +2,8 @@
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT License.
 # ------------------------------------
+import json
+
 from requests import Request, Session
 
 from ._client_factory import HTTPClientFactory
@@ -25,9 +27,10 @@ supported_options = [
 ]
 
 
-def attach_context(func):
-    """Attaches a request context object to every graph request"""
+def collect_options(func):
+    """Collect middleware options into a middleware control dict and pass it as a header"""
     def wrapper(*args, **kwargs):
+
         middleware_control = dict()
 
         for option in supported_options:
@@ -35,13 +38,12 @@ def attach_context(func):
             if value:
                 middleware_control.update({option: value})
 
-        headers = kwargs.get('headers', {})
-        request_context = RequestContext(middleware_control, headers)
+        if 'headers' in kwargs.keys():
+            kwargs['headers'].update({'middleware_control': json.dumps(middleware_control)})
+        else:
+            kwargs['headers'] = {'middleware_control': json.dumps(middleware_control)}
 
-        request = func(*args, **kwargs)
-        request.context = request_context
-
-        return request
+        return func(*args, **kwargs)
 
     return wrapper
 
@@ -81,16 +83,34 @@ class GraphClient:
         """
         self.graph_session = self.get_graph_session(**kwargs)
 
+    @collect_options
     def get(self, url: str, **kwargs):
         r"""Sends a GET request. Returns :class:`Response` object.
         :param url: URL for the new :class:`Request` object.
         :param \*\*kwargs: Optional arguments that ``request`` takes.
         :rtype: requests.Response
         """
-        prepared_request = self.prepare_request('GET', self._graph_url(url), **kwargs)
-        return self.graph_session.send(prepared_request)
+        return self.graph_session.get(self._graph_url(url), **kwargs)
 
-    def post(self, url, **kwargs):
+    def options(self, url, **kwargs):
+        r"""Sends a OPTIONS request. Returns :class:`Response` object.
+        :param url: URL for the new :class:`Request` object.
+        :param \*\*kwargs: Optional arguments that ``request`` takes.
+        :rtype: requests.Response
+        """
+
+        return self.graph_session.options(self._graph_url(url), **kwargs)
+
+    def head(self, url, **kwargs):
+        r"""Sends a HEAD request. Returns :class:`Response` object.
+        :param url: URL for the new :class:`Request` object.
+        :param \*\*kwargs: Optional arguments that ``request`` takes.
+        :rtype: requests.Response
+        """
+
+        return self.graph_session.head(self._graph_url(url), **kwargs)
+
+    def post(self, url, data=None, json=None, **kwargs):
         r"""Sends a POST request. Returns :class:`Response` object.
         :param url: URL for the new :class:`Request` object.
         :param data: (optional) Dictionary, list of tuples, bytes, or file-like
@@ -99,8 +119,7 @@ class GraphClient:
         :param \*\*kwargs: Optional arguments that ``request`` takes.
         :rtype: requests.Response
         """
-        prepared_request = self.prepare_request('POST', self._graph_url(url), **kwargs)
-        return self.graph_session.send(prepared_request)
+        return self.graph_session.post(self._graph_url(url), data=data, json=json, **kwargs)
 
     def put(self, url, data=None, **kwargs):
         r"""Sends a PUT request. Returns :class:`Response` object.
@@ -110,8 +129,8 @@ class GraphClient:
         :param \*\*kwargs: Optional arguments that ``request`` takes.
         :rtype: requests.Response
         """
-        prepared_request = self.prepare_request('PUT', self._graph_url(url), **kwargs)
-        return self.graph_session.send(prepared_request)
+
+        return self.graph_session.put(self._graph_url(url), data=data, **kwargs)
 
     def patch(self, url, data=None, **kwargs):
         r"""Sends a PATCH request. Returns :class:`Response` object.
@@ -121,8 +140,7 @@ class GraphClient:
         :param \*\*kwargs: Optional arguments that ``request`` takes.
         :rtype: requests.Response
         """
-        prepared_request = self.prepare_request('PATCH', self._graph_url(url), **kwargs)
-        return self.graph_session.send(prepared_request)
+        return self.graph_session.patch(self._graph_url(url), data=data, **kwargs)
 
     def delete(self, url, **kwargs):
         r"""Sends a DELETE request. Returns :class:`Response` object.
@@ -130,8 +148,7 @@ class GraphClient:
         :param \*\*kwargs: Optional arguments that ``request`` takes.
         :rtype: requests.Response
         """
-        prepared_request = self.prepare_request('DELETE', self._graph_url(url), **kwargs)
-        return self.graph_session.send(prepared_request)
+        return self.graph_session.delete(self._graph_url(url), **kwargs)
 
     def _graph_url(self, url: str) -> str:
         """Appends BASE_URL to user provided path
@@ -140,12 +157,6 @@ class GraphClient:
         """
         return self.graph_session.base_url + url if (url[0] == '/') else url
 
-    @attach_context
-    def prepare_request(self, method, url, **kwargs):
-        req = Request(method, url, **kwargs)
-        prepared = Session().prepare_request(req)
-        return prepared
-
     @staticmethod
     def get_graph_session(**kwargs):
         """Method to always return a single instance of a HTTP Client"""
